@@ -1,5 +1,5 @@
 /*
- * Modifications Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
  * Original X11 Port Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -31,6 +31,8 @@
 #include "resources.h"
 
 #include "ctlrc.h"
+#include "ft_cutc.h"
+#include "ftc.h"
 #include "kybdc.h"
 #include "macrosc.h"
 #include "popupsc.h"
@@ -1358,9 +1360,21 @@ Boolean erase;
 void
 ps_process()
 {
+	/* Process typeahead. */
 	while (run_ta())
 		;
+
+	/* Process pending scripts and macros. */
 	sms_continue();
+
+	/* Process file transfers. */
+	if (ft_state != FT_NONE &&	/* transfer in progress */
+	    formatted &&		/* screen is formatted */
+	    !screen_alt &&		/* 24x80 screen */
+	    !kybdlock &&		/* keyboard not locked */
+	    				/* magic field */
+	    IS_FA(screen_buf[1919]) && FA_IS_SKIP(screen_buf[1919]))
+		ft_cut_data();
 }
 
 /*
@@ -1511,8 +1525,9 @@ unsigned char	color;
 }
 
 /*
- * Copy a block of characters in the 3270 buffer, optionally including the
- * extended attributes.
+ * Copy a block of characters in the 3270 buffer, optionally including all of
+ * the extended attributes.  (The character set, which is actually kept in the
+ * extended attributes, is considered part of the characters here.)
  */
 void
 ctlr_bcopy(baddr_from, baddr_to, count, move_ea)
@@ -1521,6 +1536,7 @@ int	baddr_to;
 int	count;
 int	move_ea;
 {
+	/* Move the characters. */
 	if (memcmp((char *) &screen_buf[baddr_from],
 	           (char *) &screen_buf[baddr_to],
 		   count)) {
@@ -1537,6 +1553,40 @@ int	move_ea;
 		if (area_is_selected(baddr_to, count))
 			(void) unselect(baddr_to, count);
 	}
+
+	/*
+	 * If we aren't supposed to move all the extended attributes, move
+	 * the character sets separately.
+	 */
+	if (!move_ea) {
+		int i;
+		int any = 0;
+		int start, end, inc;
+
+		if (baddr_to < baddr_from || baddr_from + count < baddr_to) {
+			/* Scan forward. */
+			start = 0;
+			end = count + 1;
+			inc = 1;
+		} else {
+			/* Scan backward. */
+			start = count - 1;
+			end = -1;
+			inc = -1;
+		}
+
+		for (i = start; i != end; i += inc) {
+			if (ea_buf[baddr_to+i].cs != ea_buf[baddr_from+i].cs) {
+				ea_buf[baddr_to+i].cs = ea_buf[baddr_from+i].cs;
+				REGION_CHANGED(baddr_to + i, baddr_to + i + 1);
+				any++;
+			}
+		}
+		if (any && area_is_selected(baddr_to, count))
+			(void) unselect(baddr_to, count);
+	}
+
+	/* Move extended attributes. */
 	if (move_ea && memcmp((char *) &ea_buf[baddr_from],
 			      (char *) &ea_buf[baddr_to],
 			      count*sizeof(struct ea))) {
