@@ -1,5 +1,5 @@
 /*
- * Modifications Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
  * Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -36,7 +36,7 @@
 #define NO_SYS_TIME_H		1
 #endif /*]*/
 
-#if defined(_IBMR2) || defined(_SEQUENT_) /*[*/
+#if defined(_IBMR2) || defined(_SEQUENT_) || defined(__QNX__) /*[*/
 #define SEPARATE_SELECT_H	1
 #endif /*]*/
 
@@ -86,12 +86,11 @@ extern char		*efontname;
 extern Boolean		error_popup_visible;
 extern Boolean		ever_3270;
 extern Boolean		exiting;
-extern Boolean		std_ds_host;
+extern Boolean		*extended_3270font;
 extern Boolean		flipped;
 extern char		*full_current_host;
 extern char		full_model_name[];
 extern Pixmap		gray;
-extern Boolean		is_olwm;
 extern Pixel		keypadbg_pixel;
 extern Boolean		*latin1_font;
 extern int		maxCOLS;
@@ -107,6 +106,7 @@ extern Window		root_window;
 extern Boolean		scroll_initted;
 extern Boolean		shifted;
 extern Boolean		*standard_font;
+extern Boolean		std_ds_host;
 extern char		*termtype;
 extern Widget		toplevel;
 extern char		*user_icon_name;
@@ -172,6 +172,9 @@ struct font_list {
 extern struct font_list *font_list;
 extern int font_count;
 
+/*   input key type */
+enum keytype { KT_STD, KT_GE };
+
 /* Shorthand macros */
 
 #define CN	((char *) NULL)
