@@ -1,5 +1,5 @@
 /*
- * Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -719,23 +719,3 @@ keypad_set_keymap()
 	XtUninstallTranslations(keypad_container);
 	set_translations(keypad_container, &keypad_t00, &keypad_t0);
 }
-
-void
-move_keypad()
-{
-	Dimension x, y, w, h;
-
-	/* olwm gets very confused, so skip it */
-	if (is_olwm)
-		return;
-
-	if (!keypad_popped)
-		return;
-
-	toplevel_geometry(&x, &y, &w, &h);
-	if (x == sm_x && y == sm_y && w == sm_w && h == sm_h)
-		return;
-
-	XtPopdown(keypad_shell);
-	popup_popup(keypad_shell, XtGrabNone);
-}
