@@ -1,5 +1,5 @@
 /*
- * Modifications Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
  * Original X11 Port Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -32,6 +32,7 @@
 #include "ansic.h"
 #include "aplc.h"
 #include "ctlrc.h"
+#include "ftc.h"
 #include "keypadc.h"
 #include "kybdc.h"
 #include "macrosc.h"
@@ -83,7 +84,6 @@ struct trans_list *temp_keymaps;	/* temporary keymap list */
 
 /* Composite key mappings. */
 
-enum keytype { KT_STD, KT_GE };
 struct akeysym {
 	KeySym keysym;
 	enum keytype keytype;
@@ -91,7 +91,7 @@ struct akeysym {
 static struct akeysym cc_first;
 static struct composite {
 	struct akeysym k1, k2;
-	KeySym translation;
+	struct akeysym translation;
 } *composites = NULL;
 static int n_composites = 0;
 
@@ -479,13 +479,17 @@ Cardinal *num_params;
 		enq_ta(Attn_action, CN, CN);
 		return;
 	}
-	status_twait();
-	mcursor_waiting();
-	insert_mode(False);
-	kybdlock_set(KL_OIA_TWAIT | KL_OIA_LOCKED, "Attn_action");
+	if (appres.attn_lock) {
+		status_twait();
+		mcursor_waiting();
+		insert_mode(False);
+		kybdlock_set(KL_OIA_TWAIT | KL_OIA_LOCKED, "Attn_action");
+	}
 	net_break();
-	ticking_start(False);
-	status_ctlr_done();
+	if (appres.attn_lock) {
+		ticking_start(False);
+		status_ctlr_done();
+	}
 }
 
 
@@ -686,25 +690,26 @@ enum iaction	cause;
 			cc_first.keysym = c;
 			cc_first.keytype = keytype;
 			composing = FIRST;
-			status_compose(True, c);
+			status_compose(True, c, keytype);
 		} else {
 			ring_bell();
 			composing = NONE;
-			status_compose(False, 0);
+			status_compose(False, 0, KT_STD);
 		}
 		return;
 	    case FIRST:
 		composing = NONE;
-		status_compose(False, 0);
+		status_compose(False, 0, KT_STD);
 		for (i = 0; i < n_composites; i++)
 			if ((ak_eq(composites[i].k1, cc_first) &&
 			     ak_eq(composites[i].k2, ak)) ||
 			    (ak_eq(composites[i].k1, ak) &&
 			     ak_eq(composites[i].k2, cc_first)))
 				break;
-		if (i < n_composites)
-			c = composites[i].translation;
-		else {
+		if (i < n_composites) {
+			c = composites[i].translation.keysym;
+			keytype = composites[i].translation.keytype;
+		} else {
 			ring_bell();
 			return;
 		}
@@ -878,7 +883,7 @@ Boolean explicit;
 			half_reset = True;
 		if (composing != NONE) {
 			composing = NONE;
-			status_compose(False, 0);
+			status_compose(False, 0, KT_STD);
 			half_reset = True;
 		}
 		if (half_reset)
@@ -903,7 +908,7 @@ Boolean explicit;
 	 * If explicit (from the keyboard), unlock the keyboard now.
 	 * Otherwise (from the host), schedule a deferred keyboard unlock.
 	 */
-	if (explicit) {
+	if (explicit || ft_state != FT_NONE) {
 		kybdlock_clr(-1, "do_reset");
 	} else if (kybdlock &
   (KL_DEFERRED_UNLOCK | KL_OIA_TWAIT | KL_OIA_LOCKED | KL_AWAITING_FIRST)) {
@@ -917,7 +922,7 @@ Boolean explicit;
 	status_reset();
 	mcursor_normal();
 	composing = NONE;
-	status_compose(False, 0);
+	status_compose(False, 0, KT_STD);
 }
 
 /*ARGSUSED*/
@@ -2139,10 +2144,14 @@ char *s;
 enum keytype *keytypep;
 {
 	KeySym k;
+	int is_ge;
 
 	if (!strncmp(s, "apl_", 4)) {
-		k = APLStringToKeysym(s);
-		*keytypep = KT_GE;
+		k = APLStringToKeysym(s, &is_ge);
+		if (is_ge)
+			*keytypep = KT_GE;
+		else
+			*keytypep = KT_STD;
 	} else {
 		k = XStringToKeysym(s);
 		*keytypep = KT_STD;
@@ -2219,7 +2228,8 @@ build_composites()
 		cp->k1.keytype = a[0];
 		cp->k2.keysym = k[1];
 		cp->k2.keytype = a[1];
-		cp->translation = k[2];
+		cp->translation.keysym = k[2];
+		cp->translation.keytype = a[2];
 		n_composites++;
 	}
 	return True;
@@ -2250,7 +2260,7 @@ Cardinal *num_params;
 
 	if (composing == NONE) {
 		composing = COMPOSE;
-		status_compose(True, 0);
+		status_compose(True, 0, KT_STD);
 	}
 }
 
@@ -2628,6 +2638,10 @@ Boolean pasting;
 			    case 'x':
 				state = BACKX;
 				break;
+			    case '\\':
+				key_ACharacter((unsigned char) c, KT_STD, ia);
+				state = BASE;
+				break;
 			    case '0': 
 			    case '1': 
 			    case '2': 
@@ -2950,3 +2964,58 @@ Cardinal *num_params;
 	status_kmap(True);
 }
 #undef TN
+
+/*
+ * Set up the cursor and input field for command input.
+ * Returns the length of the input field, or 0 if there is no field
+ * to set up.
+ */
+int
+kybd_prime()
+{
+	int baddr;
+	register unsigned char *fa;
+	int len = 0;
+
+	/*
+	 * No point in trying if the screen isn't formatted, the keyboard
+	 * is locked, or we aren't in 3270 mode.
+	 */
+	if (!formatted || kybdlock || !IN_3270)
+		return 0;
+
+	fa = get_field_attribute(cursor_addr);
+	if (IS_FA(screen_buf[cursor_addr]) || FA_IS_PROTECTED(*fa)) {
+		/*
+		 * The cursor is not in an unprotected field.  Find the
+		 * next one.
+		 */
+		baddr = next_unprotected(cursor_addr);
+
+		/* If there isn't any, give up. */
+		if (!baddr)
+			return 0;
+
+		/* Move the cursor there. */
+	} else {
+		/* Already in an unprotected field.  Find its start. */
+		baddr = cursor_addr;
+		while (!IS_FA(screen_buf[baddr])) {
+			DEC_BA(baddr);
+		}
+		INC_BA(baddr);
+	}
+
+	/* Move the cursor to the beginning of the field. */
+	cursor_move(baddr);
+
+	/* Erase it. */
+	while (!IS_FA(screen_buf[baddr])) {
+		ctlr_add(baddr, 0, 0);
+		len++;
+		INC_BA(baddr);
+	}
+
+	/* Return the field length. */
+	return len;
+}
