@@ -1,5 +1,5 @@
 /*
- * Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -68,7 +68,7 @@ typedef struct sms {
 		ST_STRING,	/* string */
 		ST_MACRO,	/* macro */
 		ST_CHILD,	/* child process */
-		ST_PEER,	/* peer (external) process */
+		ST_PEER		/* peer (external) process */
 	} type;
 	Boolean	success;
 	Boolean	need_prompt;
@@ -1747,9 +1747,9 @@ Cardinal *num_params;
 
 		/* Export the names of the pipes into the environment. */
 		(void) sprintf(env_buf[0], "X3270OUTPUT=%d", outpipe[0]);
-		putenv(env_buf[0]);
+		(void) putenv(env_buf[0]);
 		(void) sprintf(env_buf[1], "X3270INPUT=%d", inpipe[1]);
-		putenv(env_buf[1]);
+		(void) putenv(env_buf[1]);
 
 		/* Set up arguments. */
 		argv = (char **)XtMalloc((*num_params + 1) * sizeof(char *));
