@@ -1,5 +1,5 @@
 /*
- * Modifications Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
  * Original X11 Port Copyright 1990 by Jeff Sparkes.
  *   Permission to use, copy, modify, and distribute this software and its
  *   documentation for any purpose and without fee is hereby granted,
@@ -33,6 +33,7 @@
 #include "actionsc.h"
 #include "ansic.h"
 #include "ctlrc.h"
+#include "ftc.h"
 #include "kybdc.h"
 #include "macrosc.h"
 #include "mainc.h"
@@ -80,7 +81,6 @@ int             model_num;
 int             ov_cols, ov_rows;
 char           *programname;
 Pixmap          gray;
-Boolean         is_olwm = False;
 XrmDatabase     rdb;
 Boolean         shifted = False;
 struct trans_list *trans_list = (struct trans_list *)NULL;
@@ -124,7 +124,6 @@ static XtErrorMsgHandler old_emh;
 static void     trap_colormaps();
 static Boolean  colormap_failure = False;
 static void     parse_set_clear();
-static Boolean  find_olwm();
 
 XrmOptionDescRec options[]= {
 	{ OptActiveIcon,DotActiveIcon,	XrmoptionNoArg,		ResTrue },
@@ -232,6 +231,8 @@ static XtResource resources[] = {
 	  offset(typeahead), XtRString, ResTrue },
 	{ ResDebugTracing, ClsDebugTracing, XtRBoolean, sizeof(Boolean),
 	  offset(debug_tracing), XtRString, ResTrue },
+	{ ResAttnLock, ClsAttnLock, XtRBoolean, sizeof(Boolean),
+	  offset(attn_lock), XtRString, ResFalse },
 	{ ResBellVolume, ClsBellVolume, XtRInt, sizeof(int),
 	  offset(bell_volume), XtRString, "0" },
 	{ ResOversize, ClsOversize, XtRString, sizeof(char *),
@@ -279,6 +280,8 @@ static XtResource resources[] = {
 	  offset(trace_dir), XtRString, "/tmp" },
 	{ ResColorScheme, ClsColorScheme, XtRString, sizeof(String),
 	  offset(color_scheme), XtRString, "default" },
+	{ ResFtCommand, ClsFtCommand, XtRString, sizeof(String),
+	  offset(ft_command), XtRString, "ind$file" },
 	{ ResMonoCase, ClsMonoCase, XtRBoolean, sizeof(Boolean),
 	  toggle_offset(MONOCASE), XtRString, ResFalse },
 	{ ResAltCursor, ClsAltCursor, XtRBoolean, sizeof(Boolean),
@@ -669,7 +672,6 @@ char	*argv[];
 
 	if (appres.char_class != CN)
 		reclass(appres.char_class);
-
 	screen_init(False, True, True);
 	if (appres.active_icon) {
 		XtVaSetValues(icon_shell,
@@ -681,9 +683,6 @@ char	*argv[];
 	protocols[1] = a_save_yourself;
 	XSetWMProtocols(display, XtWindow(toplevel), protocols, 2);
 
-	/* Identify the window manager. */
-	is_olwm = find_olwm();
-
 	/* Save the command line. */
 	save_init(argc, argv[1], argv[2]);
 
@@ -949,6 +948,9 @@ Boolean disable;
 
 		/* Cancel all login strings, macros, and scripts. */
 		sms_cancel_login();
+
+		/* Cancel any file transfer in progress. */
+		ft_disconnected();
 	}
 	menubar_connect();
 	kybd_connect();
@@ -978,6 +980,7 @@ Boolean now3270;
 		screen_connect();
 		relabel();
 		ansi_init();
+		ft_not3270();
 	}
 }
 
@@ -1357,32 +1360,6 @@ char **argv;
 	XtFree((XtPointer)argv_out);
 }
 
-/*
- * Find out if our window manager is olwm.
- */
-static Boolean
-find_olwm()
-{
-	Atom a;
-	Atom actual_type;
-	int actual_format;
-	unsigned long nitems;
-	unsigned long bytes_after;
-	unsigned char *prop;
-
-	a = XInternAtom(display, "_SUN_WM_PROTOCOLS", True);
-	if (a == None)
-		return False;
-
-	if (XGetWindowProperty(display, root_window, a, 0L, 1L, False,
-		    AnyPropertyType, &actual_type, &actual_format, &nitems,
-		    &bytes_after, &prop) == Success) {
-		XFree(prop);
-		return (nitems > 0) ? True : False;
-	}
-	return False;
-}
-
 /*
  * Safe routine for querying window attributes
  */
