@@ -1,5 +1,5 @@
 /*
- * Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -25,6 +25,7 @@
 
 #include "actionsc.h"
 #include "aboutc.h"
+#include "ftc.h"
 #include "keypadc.h"
 #include "kybdc.h"
 #include "macrosc.h"
@@ -74,12 +75,14 @@ static void	keypad_button_init();
 static void	connect_menu_init();
 static void	macros_menu_init();
 static void	file_menu_init();
+static void	Bye();
 
 #include "dot.bm"
 #include "arrow.bm"
 #include "diamond.bm"
 #include "no_diamond.bm"
 #include "ky.bm"
+#include "null.bm"
 
 Boolean keypad_changed = False;
 
@@ -92,7 +95,9 @@ static Widget   menu_bar;
 static Boolean  menubar_buttons;
 static Widget   disconnect_button;
 static Widget   exit_button;
+static Widget   exit_menu;
 static Widget   macros_button;
+static Widget   ft_button;
 static Widget   connect_button;
 static Widget   keypad_button;
 static Widget   linemode_button;
@@ -108,10 +113,13 @@ static Widget	script_abort_button;
 static Widget   connect_menu;
 static Widget   macros_menu;
 
-static Pixmap   dot;
 static Pixmap   arrow;
-static Pixmap	diamond;
-static Pixmap	no_diamond;
+Pixmap dot;
+Pixmap diamond;
+Pixmap no_diamond;
+Pixmap null;
+
+static int	n_bye;
 
 static void     toggle_init();
 
@@ -125,7 +133,6 @@ static void     toggle_init();
 
 #define MO_OFFSET	1
 #define CN_OFFSET	2
-
 #define MENU_BORDER	2
 
 #define	MENU_MIN_WIDTH	(LEFT_MARGIN + 3*(KEY_WIDTH+2*BORDER+SPACING) + \
@@ -195,10 +202,12 @@ Dimension current_width;
 	    (char *) diamond_bits, diamond_width, diamond_height);
 	no_diamond = XCreateBitmapFromData(display, root_window,
 	    (char *) no_diamond_bits, no_diamond_width, no_diamond_height);
+	null = XCreateBitmapFromData(display, root_window,
+	    (char *) null_bits, null_width, null_height);
 
 	/* "File..." menu */
 
-	file_menu_init(menu_bar, 
+	file_menu_init(menu_bar,
 	    LEFT_MARGIN,
 	    TOP_MARGIN);
 
@@ -244,10 +253,35 @@ menubar_connect()	/* have connected to or disconnected from a host */
 		XtVaSetValues(disconnect_button,
 		    XtNsensitive, PCONNECTED,
 		    NULL);
-	if (exit_button)
-		XtVaSetValues(exit_button,
-		    XtNsensitive, !PCONNECTED,
-		    NULL);
+	if (exit_button) {
+		if (PCONNECTED) {
+			/* Remove the immediate callback. */
+			if (n_bye) {
+				XtRemoveCallback(exit_button, XtNcallback,
+				    Bye, 0);
+				n_bye--;
+			}
+
+			/* Set pullright for extra confirmation. */
+			XtVaSetValues(exit_button,
+			    XtNrightBitmap, arrow,
+			    XtNmenuName, "exitMenu",
+			    NULL);
+		} else {
+			/* Install the immediate callback. */
+			if (!n_bye) {
+				XtAddCallback(exit_button, XtNcallback,
+				    Bye, 0);
+				n_bye++;
+			}
+
+			/* Remove the pullright. */
+			XtVaSetValues(exit_button,
+			    XtNrightBitmap, NULL,
+			    XtNmenuName, NULL,
+			    NULL);
+		}
+	}
 	if (connect_menu) {
 		if (PCONNECTED && connect_button)
 			XtUnmapWidget(connect_button);
@@ -269,6 +303,8 @@ menubar_connect()	/* have connected to or disconnected from a host */
 		if (menubar_buttons && macros_button)
 			XtMapWidget(macros_button);
 	}
+	if (ft_button)
+		XtVaSetValues(ft_button, XtNsensitive, IN_3270, NULL);
 	if (linemode_button)
 		XtVaSetValues(linemode_button, XtNsensitive, IN_ANSI, NULL);
 	if (charmode_button)
@@ -316,6 +352,8 @@ menubar_newmode()	/* have switched telnet modes */
 	if (appres.toggle[RECTANGLE_SELECT].w[0])
 		XtVaSetValues(appres.toggle[RECTANGLE_SELECT].w[0],
 		    XtNsensitive, IN_ANSI, NULL);
+	if (ft_button)
+		XtVaSetValues(ft_button, XtNsensitive, IN_3270, NULL);
 }
 
 void
@@ -450,6 +488,18 @@ Dimension x, y;
 	    NULL);
 	XtAddCallback(w, XtNcallback, show_about, NULL);
 
+	/* File Transfer */
+	if (!appres.secure) {
+		(void) XtCreateManagedWidget(
+		    "space", cmeLineObjectClass, file_menu,
+		    NULL, 0);
+		ft_button = XtVaCreateManagedWidget(
+		    "ftOption", cmeBSBObjectClass, file_menu,
+		    XtNsensitive, IN_3270,
+		    NULL);
+		XtAddCallback(ft_button, XtNcallback, popup_ft, NULL);
+	}
+
 	/* Trace Data Stream
 	   Trace X Events
 	   Save Screen(s) in File */
@@ -520,11 +570,20 @@ Dimension x, y;
 	XtAddCallback(disconnect_button, XtNcallback, disconnect, 0);
 
 	/* Exit x3270 */
+	if (exit_menu != (Widget)NULL)
+		XtDestroyWidget(exit_menu);
+	exit_menu = XtVaCreatePopupShell(
+	    "exitMenu", complexMenuWidgetClass, container,
+	    NULL);
+	w = XtVaCreateManagedWidget(
+	    "exitReallyOption", cmeBSBObjectClass, exit_menu,
+	    NULL);
+	XtAddCallback(w, XtNcallback, Bye, 0);
 	exit_button = XtVaCreateManagedWidget(
 	    "exitOption", cmeBSBObjectClass, file_menu,
-	    XtNsensitive, !PCONNECTED,
 	    NULL);
 	XtAddCallback(exit_button, XtNcallback, Bye, 0);
+	n_bye = 1;
 
 	/* File... */
 	if (menubar_buttons) {
@@ -643,7 +702,7 @@ Position x, y;
 		buf = xs_buffer("%s %s", get_message("reconnect"),
 		    current_host);
 		w = XtVaCreateManagedWidget(
-		    buf, cmeBSBObjectClass, connect_menu, 
+		    buf, cmeBSBObjectClass, connect_menu,
 		    NULL);
 		XtFree(buf);
 		XtAddCallback(w, XtNcallback, do_reconnect, PN);
@@ -663,7 +722,7 @@ Position x, y;
 			    cmeLineObjectClass, connect_menu, NULL, 0);
 		any_hosts = True;
 		w = XtVaCreateManagedWidget(
-		    h->name, cmeBSBObjectClass, connect_menu, 
+		    h->name, cmeBSBObjectClass, connect_menu,
 		    NULL);
 		XtAddCallback(w, XtNcallback, host_connect,
 		    XtNewString(h->name));
@@ -778,7 +837,7 @@ Position x, y;
 				    cmeLineObjectClass, macros_menu, NULL, 0);
 		}
 		w = XtVaCreateManagedWidget(
-		    m->name, cmeBSBObjectClass, macros_menu, 
+		    m->name, cmeBSBObjectClass, macros_menu,
 		    NULL);
 		XtAddCallback(w, XtNcallback, do_macro, (XtPointer)m);
 		any = True;
@@ -815,7 +874,7 @@ XtPointer call_data;
 	    case kp_right:
 	    case kp_bottom:
 		keypad_popup_init();
-		if (keypad_popped) 
+		if (keypad_popped)
 			XtPopdown(keypad_shell);
 		else
 			popup_popup(keypad_shell, XtGrabNone);
@@ -962,7 +1021,7 @@ Widget w;
 XtPointer userdata;
 XtPointer calldata;
 {
-	(void) screen_newfont((char *)userdata, True);
+	screen_newfont((char *)userdata, True);
 }
 
 /* Called from the "Select Font" button on the font dialog */
@@ -1142,7 +1201,7 @@ XtPointer call_data;
 {
 	net_charmode();
 }
- 
+
 /* Called to change models */
 /*ARGSUSED*/
 static void
