@@ -1,5 +1,5 @@
 /*
- * Modifications Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
  * Original X11 Port Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -71,7 +71,6 @@ extern int      last_changed;
 /* Globals */
 unsigned char  *selected;	/* selection bitmap */
 Dimension       main_width, main_height;
-Position        main_x, main_y;
 #include <X11/bitmaps/gray>
 Boolean         scrollbar_changed = False;
 Boolean         model_changed = False;
@@ -122,9 +121,21 @@ static XtIntervalId text_blink_id;
 static XtTranslations screen_t00 = NULL;
 static XtTranslations screen_t0 = NULL;
 static XtTranslations container_t0 = NULL;
-static char    *rt_buf8 = (char *) NULL;
+static unsigned char *rt_buf8 = (unsigned char *) NULL;
 static XChar2b *rt_buf16 = (XChar2b *) NULL;
 static char    *color_name[16];
+static Boolean	configure_ticking = False;
+static XtIntervalId configure_id;
+static Boolean	configure_first = True;
+
+/* Globals for undoing reconfigurations. */
+static enum {
+    REDO_NONE, REDO_FONT, REDO_MODEL, REDO_KEYPAD, REDO_SCROLLBAR
+} screen_redo = REDO_NONE;
+static char *redo_old_font = CN;
+static int redo_old_model;
+static int redo_old_ov_cols;
+static int redo_old_ov_rows;
 
 static unsigned char blank_map[32];
 #define BKM_SET(n)	blank_map[(n)/8] |= 1 << ((n)%8)
@@ -164,6 +175,7 @@ static GC       get_gc();
 static GC       get_selgc();
 static void	default_color_scheme();
 static Boolean	xfer_color_scheme();
+static void	configure_stable();
 
 /* Resize font list. */
 struct rsfont {
@@ -220,6 +232,7 @@ XFontStruct   **efontinfo = &nss.efontinfo;
 Boolean        *standard_font = &nss.standard_font;
 Boolean        *latin1_font = &nss.latin1_font;
 Boolean        *debugging_font = &nss.debugging_font;
+Boolean        *extended_3270font = &nss.extended_3270font;
 
 #define BASE_MASK		0x0f	/* mask for 16 actual colors */
 #define INVERT_MASK		0x10	/* toggle for inverted colors */
@@ -399,9 +412,9 @@ Boolean font_changed;
 		temp_image = (union sp *) XtCalloc(sizeof(union sp), maxROWS*maxCOLS);
 
 		/* render_text buffers */
-		if (rt_buf8 != CN)
+		if (rt_buf8 != (unsigned char *)NULL)
 			XtFree((char *)rt_buf8);
-		rt_buf8 = XtMalloc(maxCOLS);
+		rt_buf8 = (unsigned char *)XtMalloc(maxCOLS);
 		if (rt_buf16 != (XChar2b *)NULL)
 			XtFree((char *)rt_buf16);
 		rt_buf16 = (XChar2b *)XtMalloc(maxCOLS * sizeof(XChar2b));
@@ -504,7 +517,15 @@ set_toplevel_sizes()
 	    NULL);
 	main_width = tw;
 	main_height = th;
-	move_keypad();
+
+	/*
+	 * Start a timer ticking, in case the window manager doesn't approve
+	 * of the change.
+	 */
+	if (configure_ticking)
+		XtRemoveTimeOut(configure_id);
+	configure_id = XtAppAddTimeOut(appcontext, 500, configure_stable, 0);
+	configure_ticking = True;
 }
 
 static void
@@ -617,9 +638,10 @@ enum toggle_type tt;
 {
 	scrollbar_changed = True;
 
-	if (toggled(SCROLL_BAR))
+	if (toggled(SCROLL_BAR)) {
 		scrollbar_width = SCROLLBAR_WIDTH;
-	else {
+		screen_redo = REDO_SCROLLBAR;
+	} else {
 		scroll_to_bottom();
 		scrollbar_width = 0;
 	}
@@ -708,6 +730,9 @@ void
 screen_showikeypad(on)
 Boolean on;
 {
+	if (on)
+		screen_redo = REDO_KEYPAD;
+
 	if (keypad_xwidth > 0) {
 		XtDestroyWidget(nss.widget);
 		nss.widget = (Widget) NULL;
@@ -1406,7 +1431,8 @@ union sp *attrs;	/* buffer to use for attributes */
 		XDrawImageString16(display, ss->window, dgc, x, y, rt_buf16,
 		    len);
 	else
-		XDrawImageString(display, ss->window, dgc, x, y, rt_buf8, len);
+		XDrawImageString(display, ss->window, dgc, x, y,
+		    (char *)rt_buf8, len);
 	if (ss->overstrike &&
 	    ((attrs->bits.gr & GR_INTENSIFY) ||
 	     (appres.mono && ((color & BASE_MASK) == FA_INT_HIGH_SEL)))) {
@@ -1415,7 +1441,7 @@ union sp *attrs;	/* buffer to use for attributes */
 			    rt_buf16, len);
 		else
 			XDrawString(display, ss->window, dgc, x+1, y,
-			    rt_buf8, len);
+			    (char *)rt_buf8, len);
 	}
 	if (attrs->bits.gr & GR_UNDERLINE)
 		XDrawLine(display,
@@ -1834,6 +1860,7 @@ Boolean invert;
 	union sp buffer;
 	unsigned char *fa;
 	int gr;
+	int blank_it = 0;
 
 	if (!invert) {
 		int flb = fl_baddr(baddr);
@@ -1850,18 +1877,23 @@ Boolean invert;
 	 */
 	buffer.word = 0L;
 	buffer.bits.cg = screen_buf[baddr];
+	buffer.bits.cs = ea_buf[baddr].cs;
 	fa = get_field_attribute(baddr);
 	gr = ea_buf[baddr].gr;
 	if (!gr)
 		gr = fa2ea(fa)->gr;
 	if (IS_FA(screen_buf[baddr])) {
 		if (!ss->debugging_font)
-			buffer.bits.cg = CG_space;
+			blank_it = 1;
 	} else if (FA_IS_ZERO(*fa)) {
-		buffer.bits.cg = CG_space;
+		blank_it = 1;
 	} else if (text_blinkers_exist && !text_blinking_on) {
 		if (gr & GR_BLINK)
-			buffer.bits.cg = CG_space;
+			blank_it = 1;
+	}
+	if (blank_it) {
+		buffer.bits.cg = CG_space;
+		buffer.bits.cs = 0;
 	}
 	buffer.bits.fg = char_color(baddr);
 	buffer.bits.gr |= (gr & GR_INTENSIFY);
@@ -2704,10 +2736,10 @@ Cardinal *num_params;
 	action_debug(SetFont_action, event, params, num_params);
 	if (check_usage(SetFont_action, *num_params, 1, 1) < 0)
 		return;
-	(void) screen_newfont(params[0], True);
+	screen_newfont(params[0], True);
 }
 
-int
+void
 screen_newfont(fontname, do_popup)
 char *fontname;
 Boolean do_popup;
@@ -2718,12 +2750,12 @@ Boolean do_popup;
 
 	/* Can't change fonts in APL mode. */
 	if (appres.apl_mode)
-		return -1;
+		return;
 
 	if (efontname && !strcmp(fontname, efontname)) {
 		if (buf)
 			XtFree(buf);
-		return 0;
+		return;
 	}
 
 	if ((emsg = load_fixed_font(fontname, &new_fontinfo))) {
@@ -2731,8 +2763,14 @@ Boolean do_popup;
 			XtFree(buf);
 		if (do_popup)
 			popup_an_error("Font \"%s\"\n%s", fontname, emsg);
-		return -1;
+		return;
 	}
+
+	if (redo_old_font != CN)
+		XtFree(redo_old_font);
+	redo_old_font = XtNewString(efontname);
+	screen_redo = REDO_FONT;
+
 	nss.efontinfo = new_fontinfo;
 	set_font_globals(nss.efontinfo, fontname);
 
@@ -2741,7 +2779,6 @@ Boolean do_popup;
 	if (buf)
 		XtFree(buf);
 	efont_changed = True;
-	return 0;
 }
 
 /*
@@ -2822,6 +2859,12 @@ int ovc, ovr;
 	if (CONNECTED ||
 	    (model_num == mn && ovc == ov_cols && ovr == ov_rows))
 		return;
+
+	redo_old_model = model_num;
+	redo_old_ov_cols = ov_cols;
+	redo_old_ov_rows = ov_rows;
+	screen_redo = REDO_MODEL;
+
 	model_changed = True;
 	if (ov_cols != ovc || ov_rows != ovr)
 		oversize_changed = True;
@@ -3213,8 +3256,6 @@ init_rsfonts()
 /*
  * Handle Configure events.
  */
-static Boolean configure_ticking = False;
-static char *configure_last_font = CN;
 
 /*
  * Timeout routine called 0.5 sec after Configure_action changes to a new
@@ -3229,7 +3270,6 @@ XtPointer closure;
 XtIntervalId *id;
 {
 	configure_ticking = False;
-	configure_last_font = CN;
 }
 
 /*ARGSUSED*/
@@ -3241,50 +3281,92 @@ String	*params;
 Cardinal *num_params;
 {
 	XConfigureEvent *re = (XConfigureEvent *) event;
-	Position xx, yy;
-	Boolean needs_moving = False;
 	int current_area;
 	int requested_area;
 	struct rsfont *r;
 	struct rsfont *smallest = (struct rsfont *) NULL;
 	struct rsfont *largest = (struct rsfont *) NULL;
 	struct rsfont *best = (struct rsfont *) NULL;
-	char *new_font;
 
 	action_debug(Configure_action, event, params, num_params);
 
-	/*
-	 * Get the new window coordinates.  If the configure event reports it
-	 * as (0,0), ask for it explicitly.
-	 */
-	if (re->x || re->y) {
-		xx = re->x;
-		yy = re->y;
-	} else {
-		XtVaGetValues(toplevel, XtNx, &xx, XtNy, &yy, NULL);
-	}
-
-	/* Save the new coordinates in globals for next time. */
-	if (xx != main_x || yy != main_y) {
-		main_x = re->x;
-		main_y = re->y;
-		needs_moving = True;
+	/* If the window dimensions are "correct", there is nothing to do. */
+	if (re->width == main_width && re->height == main_height) {
+		configure_first = False;
+		return;
 	}
 
 	/*
-	 * If the window dimensions are unchanged, there is nothing more to
-	 * do but relocate the keypad.
+	 * There are three possible reasons for being here:
+	 * (1) The user has (through the window manager) resized our
+	 *  window.
+	 * (2) We resized our window when our configuration changed (font,
+	 *  model, integral keypad or scrollbar); mwm thinks it is too big
+	 *  and scaled it back.
+	 * (3) Mwm thinks our initial screen size is too big, and scaled it
+	 *  back.
+	 *
+	 * 'configure_ticking' and the associated timeout attempt to detect
+	 * (2) and (3).  Whenever we set the toplevel dimensions, we set
+	 * the timer; if a changed configure comes in while the timer is
+	 * ticking, we assume it is an angry mwm.  If this is the very first
+	 * time we've sized the window, we must crash; otherwise, we can
+	 * simply back off to the previous configuration.
+	 *
+	 * If 'configure_ticking' is clear, we assume the cause is (1).
+	 * If resize fonts are available, we pick the best one and try it.
+	 * Otherwise, we restore the original toplevel dimensions.
 	 */
-	if (re->width == main_width && re->height == main_height)
-		goto done;
+
+	/* If this was our first resize, crash. */
+	if (configure_first)
+		xs_error("This x3270 configuration does not fit on the X display.");
+
+	if (configure_ticking) {
+		/*
+		 * We must presume this is the window manager disapproving of
+		 * our last resize attempt.
+		 */
+		XtRemoveTimeOut(configure_id);
+		configure_ticking = False;
+		popup_an_error("This configuration does not fit on the X display");
+
+		/* Restore the old configuration... */
+		switch (screen_redo) {
+		    case REDO_FONT:
+			screen_newfont(redo_old_font, False);
+			break;
+		    case REDO_MODEL:
+			screen_change_model(redo_old_model,
+			    redo_old_ov_cols, redo_old_ov_rows);
+			break;
+		    case REDO_KEYPAD:
+			screen_showikeypad(appres.keypad_on = False);
+			break;
+		    case REDO_SCROLLBAR:
+			if (toggled(SCROLL_BAR)) {
+				toggle_toggle(&appres.toggle[SCROLL_BAR]);
+				toggle_scrollBar(&appres.toggle[SCROLL_BAR],
+				    TT_TOGGLE);
+			}
+			break;
+		    case REDO_NONE:
+		    default:
+			xs_error("Internal reconfiguration error.");
+			break;
+		}
+		screen_redo = REDO_NONE;
+
+		return;
+	}
 
 	/*
-	 * If the resize font list is empty, we can't resize the screen.
-	 * Just restore the previous window size and relocate the keypad.
+	 * User-generated resize attempt.
 	 */
 	if (rsfonts == (struct rsfont *)NULL || !appres.allow_resize) {
+		/* Illegal or impossible. */
 		set_toplevel_sizes();
-		goto done;
+		return;
 	}
 
 	/*
@@ -3326,7 +3408,7 @@ Cardinal *num_params;
 	requested_area = re->width * re->height;
 	if (current_area == requested_area) {
 		set_toplevel_sizes();
-		goto done;
+		return;
 	}
 	smallest = (struct rsfont *) NULL;
 	largest = (struct rsfont *) NULL;
@@ -3376,25 +3458,8 @@ Cardinal *num_params;
 			    " switching to font '%s' (%dx%d)\n",
 			    best->name, best->total_width, best->total_height);
 		}
-		if (configure_ticking) {
-			popup_an_error(
-"Model %d, font '%s' will not fit on the display\n\
-Switching to font '%s'",
-			    model_num, configure_last_font, best->name);
-		} else {
-			(void) XtAppAddTimeOut(appcontext, 500,
-			    configure_stable, 0);
-			configure_ticking = True;
-			configure_last_font = best->name;
-		}
-		(void) screen_newfont(best->name, False);
+		screen_newfont(best->name, False);
 	}
-
-	return;
-
-    done:
-	if (needs_moving)
-		move_keypad();
 }
 
 /*
